@@ -132,12 +132,17 @@ static void gtk_menu_item_buildable_add_child      (GtkBuildable        *buildab
 						    GtkBuilder          *builder,
 						    GObject             *child,
 						    const gchar         *type);
+static void gtk_menu_item_buildable_custom_finished(GtkBuildable        *buildable,
+						    GtkBuilder          *builder,
+						    GObject             *child,
+						    const gchar         *tagname,
+						    gpointer             user_data);
 
 static void gtk_menu_item_activatable_interface_init (GtkActivatableIface  *iface);
-static void gtk_menu_item_activatable_update         (GtkActivatable       *activatable,
+static void gtk_menu_item_update                     (GtkActivatable       *activatable,
 						      GtkAction            *action,
 						      const gchar          *property_name);
-static void gtk_menu_item_activatable_reset          (GtkActivatable       *activatable,
+static void gtk_menu_item_sync_action_properties     (GtkActivatable       *activatable,
 						      GtkAction            *action);
 static void gtk_menu_item_set_related_action         (GtkMenuItem          *menu_item, 
 						      GtkAction            *action);
@@ -558,6 +563,7 @@ gtk_menu_item_buildable_interface_init (GtkBuildableIface *iface)
 {
   parent_buildable_iface = g_type_interface_peek_parent (iface);
   iface->add_child = gtk_menu_item_buildable_add_child;
+  iface->custom_finished = gtk_menu_item_buildable_custom_finished;
 }
 
 static void 
@@ -573,18 +579,58 @@ gtk_menu_item_buildable_add_child (GtkBuildable *buildable,
     parent_buildable_iface->add_child (buildable, builder, child, type);
 }
 
+
+static void 
+gtk_menu_item_buildable_custom_finished (GtkBuildable        *buildable,
+					 GtkBuilder          *builder,
+					 GObject             *child,
+					 const gchar         *tagname,
+					 gpointer             user_data)
+{
+  GtkWidget *toplevel;
+
+  if (strcmp (tagname, "accelerator") == 0)
+    {
+      GtkMenuShell *menu_shell = (GtkMenuShell *) GTK_WIDGET (buildable)->parent;
+      GtkWidget *attach;
+
+      if (menu_shell)
+	{
+	  while (GTK_IS_MENU (menu_shell) &&
+		 (attach = gtk_menu_get_attach_widget (GTK_MENU (menu_shell))) != NULL)
+	    menu_shell = (GtkMenuShell *)attach->parent;
+	  
+	  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (menu_shell));
+	}
+      else
+	{
+	  /* Fall back to something ... */
+	  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (buildable));
+
+	  g_warning ("found a GtkMenuItem '%s' without a parent GtkMenuShell, assigned accelerators wont work.",
+		     gtk_buildable_get_name (buildable));
+	}
+
+      /* Feed the correct toplevel to the GtkWidget accelerator parsing code */
+      _gtk_widget_buildable_finish_accelerator (GTK_WIDGET (buildable), toplevel, user_data);
+    }
+  else
+    parent_buildable_iface->custom_finished (buildable, builder, child, tagname, user_data);
+}
+
+
 static void
 gtk_menu_item_activatable_interface_init (GtkActivatableIface *iface)
 {
-  iface->update = gtk_menu_item_activatable_update;
-  iface->reset = gtk_menu_item_activatable_reset;
+  iface->update = gtk_menu_item_update;
+  iface->sync_action_properties = gtk_menu_item_sync_action_properties;
 }
 
-static void 
+static void
 activatable_update_label (GtkMenuItem *menu_item, GtkAction *action)
 {
   GtkWidget *child = GTK_BIN (menu_item)->child;
-	  
+
   if (GTK_IS_LABEL (child))
     {
       const gchar *label;
@@ -597,9 +643,9 @@ activatable_update_label (GtkMenuItem *menu_item, GtkAction *action)
 gboolean _gtk_menu_is_empty (GtkWidget *menu);
 
 static void
-gtk_menu_item_activatable_update (GtkActivatable *activatable,
-				  GtkAction      *action,
-				  const gchar    *property_name)
+gtk_menu_item_update (GtkActivatable *activatable,
+		      GtkAction      *action,
+		      const gchar    *property_name)
 {
   GtkMenuItem *menu_item = GTK_MENU_ITEM (activatable);
   GtkMenuItemPrivate *priv = GET_PRIVATE (menu_item);
@@ -617,8 +663,8 @@ gtk_menu_item_activatable_update (GtkActivatable *activatable,
 }
 
 static void
-gtk_menu_item_activatable_reset (GtkActivatable *activatable,
-				 GtkAction      *action)
+gtk_menu_item_sync_action_properties (GtkActivatable *activatable,
+				      GtkAction      *action)
 {
   GtkMenuItem *menu_item = GTK_MENU_ITEM (activatable);
   GtkMenuItemPrivate *priv = GET_PRIVATE (menu_item);
@@ -626,7 +672,7 @@ gtk_menu_item_activatable_reset (GtkActivatable *activatable,
   if (!action)
     return;
 
-  _gtk_action_sync_menu_visible (action, GTK_WIDGET (menu_item), 
+  _gtk_action_sync_menu_visible (action, GTK_WIDGET (menu_item),
 				 _gtk_menu_is_empty (gtk_menu_item_get_submenu (menu_item)));
 
   gtk_widget_set_sensitive (GTK_WIDGET (menu_item), gtk_action_is_sensitive (action));
@@ -634,14 +680,14 @@ gtk_menu_item_activatable_reset (GtkActivatable *activatable,
   if (priv->use_action_appearance)
     {
       GtkWidget *label = GTK_BIN (menu_item)->child;
-	  
+
       /* make sure label is a label */
       if (label && !GTK_IS_LABEL (label))
 	{
 	  gtk_container_remove (GTK_CONTAINER (menu_item), label);
 	  label = NULL;
 	}
-	  
+
       if (!label)
 	label = g_object_new (GTK_TYPE_ACCEL_LABEL,
 			      "use-underline", TRUE,
@@ -649,18 +695,18 @@ gtk_menu_item_activatable_reset (GtkActivatable *activatable,
 			      "visible", TRUE,
 			      "parent", menu_item,
 			      NULL);
-      
+
       if (GTK_IS_ACCEL_LABEL (label) && gtk_action_get_accel_path (action))
 	g_object_set (label,
 		      "accel-closure", gtk_action_get_accel_closure (action),
 		      NULL);
-      
+
       activatable_update_label (menu_item, action);
     }
 }
 
 static void
-gtk_menu_item_set_related_action (GtkMenuItem *menu_item, 
+gtk_menu_item_set_related_action (GtkMenuItem *menu_item,
 				  GtkAction   *action)
 {
     GtkMenuItemPrivate *priv = GET_PRIVATE (menu_item);
@@ -691,7 +737,7 @@ gtk_menu_item_set_related_action (GtkMenuItem *menu_item,
 }
 
 static void
-gtk_menu_item_set_use_action_appearance (GtkMenuItem *menu_item, 
+gtk_menu_item_set_use_action_appearance (GtkMenuItem *menu_item,
 					 gboolean     use_appearance)
 {
     GtkMenuItemPrivate *priv = GET_PRIVATE (menu_item);
@@ -699,8 +745,8 @@ gtk_menu_item_set_use_action_appearance (GtkMenuItem *menu_item,
     if (priv->use_action_appearance != use_appearance)
       {
 	priv->use_action_appearance = use_appearance;
-	
-	gtk_activatable_reset (GTK_ACTIVATABLE (menu_item), priv->action);
+
+	gtk_activatable_sync_action_properties (GTK_ACTIVATABLE (menu_item), priv->action);
       }
 }
 
@@ -708,7 +754,7 @@ gtk_menu_item_set_use_action_appearance (GtkMenuItem *menu_item,
 /**
  * gtk_menu_item_set_submenu:
  * @menu_item: a #GtkMenuItem
- * @submenu: the submenu, or %NULL
+ * @submenu: (allow-none): the submenu, or %NULL
  *
  * Sets or replaces the menu item's submenu, or removes it when a %NULL
  * submenu is passed.
@@ -1328,6 +1374,9 @@ static gboolean
 gtk_menu_item_mnemonic_activate (GtkWidget *widget,
 				 gboolean   group_cycling)
 {
+  if (GTK_IS_MENU_SHELL (widget->parent))
+    _gtk_menu_shell_set_keyboard_mode (GTK_MENU_SHELL (widget->parent), TRUE);
+
   if (group_cycling &&
       widget->parent &&
       GTK_IS_MENU_SHELL (widget->parent) &&
@@ -1955,7 +2004,7 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
 /**
  * gtk_menu_item_set_accel_path
  * @menu_item:  a valid #GtkMenuItem
- * @accel_path: accelerator path, corresponding to this menu item's
+ * @accel_path: (allow-none): accelerator path, corresponding to this menu item's
  *              functionality, or %NULL to unset the current path.
  *
  * Set the accelerator path on @menu_item, through which runtime changes of the
