@@ -284,10 +284,10 @@ static gboolean recent_view_query_tooltip_cb      (GtkWidget        *widget,
                                                    gpointer          user_data);
 
 static void gtk_recent_chooser_activatable_iface_init (GtkActivatableIface  *iface);
-static void gtk_recent_chooser_activatable_update     (GtkActivatable       *activatable,
+static void gtk_recent_chooser_update                 (GtkActivatable       *activatable,
 						       GtkAction            *action,
 						       const gchar          *property_name);
-static void gtk_recent_chooser_activatable_reset      (GtkActivatable       *activatable,
+static void gtk_recent_chooser_sync_action_properties (GtkActivatable       *activatable,
 						       GtkAction            *action);
 
 G_DEFINE_TYPE_WITH_CODE (GtkRecentChooserDefault,
@@ -318,12 +318,12 @@ gtk_recent_chooser_iface_init (GtkRecentChooserIface *iface)
   iface->list_filters = gtk_recent_chooser_default_list_filters;
 }
 
-static void 
-gtk_recent_chooser_activatable_iface_init (GtkActivatableIface  *iface)
+static void
+gtk_recent_chooser_activatable_iface_init (GtkActivatableIface *iface)
 
-{  
-  iface->update = gtk_recent_chooser_activatable_update;
-  iface->reset  = gtk_recent_chooser_activatable_reset;
+{
+  iface->update = gtk_recent_chooser_update;
+  iface->sync_action_properties = gtk_recent_chooser_sync_action_properties;
 }
 
 static void
@@ -724,7 +724,7 @@ get_toplevel (GtkWidget *widget)
   GtkWidget *toplevel;
 
   toplevel = gtk_widget_get_toplevel (widget);
-  if (!GTK_WIDGET_TOPLEVEL (toplevel))
+  if (!gtk_widget_is_toplevel (toplevel))
     return NULL;
   else
     return GTK_WINDOW (toplevel);
@@ -1676,6 +1676,7 @@ copy_activated_cb (GtkMenuItem *menu_item,
 			  			    GDK_SELECTION_CLIPBOARD),
                           utf8_uri, -1);
 
+  gtk_recent_info_unref (info);
   g_free (utf8_uri);
 }
 
@@ -1948,10 +1949,10 @@ set_recent_manager (GtkRecentChooserDefault *impl,
     }
 }
 
-static void 
-gtk_recent_chooser_activatable_update (GtkActivatable       *activatable,
-				       GtkAction            *action,
-				       const gchar          *property_name)
+static void
+gtk_recent_chooser_update (GtkActivatable *activatable,
+			   GtkAction      *action,
+			   const gchar    *property_name)
 {
   if (strcmp (property_name, "visible") == 0)
     {
@@ -1964,13 +1965,13 @@ gtk_recent_chooser_activatable_update (GtkActivatable       *activatable,
   if (strcmp (property_name, "sensitive") == 0)
     gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
 
-  _gtk_recent_chooser_activatable_update (activatable, action, property_name);
+  _gtk_recent_chooser_update (activatable, action, property_name);
 }
 
 
 static void 
-gtk_recent_chooser_activatable_reset (GtkActivatable       *activatable,
-				      GtkAction            *action)
+gtk_recent_chooser_sync_action_properties (GtkActivatable *activatable,
+				           GtkAction      *action)
 {
   if (action)
     {
@@ -1982,7 +1983,7 @@ gtk_recent_chooser_activatable_reset (GtkActivatable       *activatable,
       gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
     }
 
-  _gtk_recent_chooser_activatable_reset (activatable, action);
+  _gtk_recent_chooser_sync_action_properties (activatable, action);
 }
 
 
