@@ -635,8 +635,8 @@ gtk_target_table_free (GtkTargetEntry *targets,
 
 /**
  * gtk_selection_owner_set_for_display:
- * @display: the #Gdkdisplay where the selection is set 
- * @widget: new selection owner (a #GdkWidget), or %NULL.
+ * @display: the #Gdkdisplay where the selection is set
+ * @widget: (allow-none): new selection owner (a #GdkWidget), or %NULL.
  * @selection: an interned atom representing the selection to claim.
  * @time_: timestamp with which to claim the selection
  *
@@ -738,7 +738,7 @@ gtk_selection_owner_set_for_display (GdkDisplay   *display,
 
 /**
  * gtk_selection_owner_set:
- * @widget:  a #GtkWidget, or %NULL.
+ * @widget: (allow-none):  a #GtkWidget, or %NULL.
  * @selection:  an interned atom representing the selection to claim
  * @time_: timestamp with which to claim the selection
  * 
@@ -1756,10 +1756,11 @@ gtk_selection_data_set_uris (GtkSelectionData  *selection_data,
  * @selection_data: a #GtkSelectionData
  * 
  * Gets the contents of the selection data as array of URIs.
- * 
- * Return value: if the selection data contains a list of
+ *
+ * Return value:  (array zero-terminated=1) (element-type utf8) (transfer full): if
+ *   the selection data contains a list of
  *   URIs, a newly allocated %NULL-terminated string array
- *   containing the URIs, otherwise %NULL. If the result is 
+ *   containing the URIs, otherwise %NULL. If the result is
  *   non-%NULL it must be freed with g_strfreev().
  *
  * Since: 2.6
@@ -1815,12 +1816,9 @@ gtk_selection_data_get_targets (GtkSelectionData  *selection_data,
 {
   g_return_val_if_fail (selection_data != NULL, FALSE);
 
-  /* As usual, java gets it wrong and sets the type to TARGETS, not ATOM 
-   */
   if (selection_data->length >= 0 &&
       selection_data->format == 32 &&
-      (selection_data->type == GDK_SELECTION_TYPE_ATOM ||
-       selection_data->type == gtk_selection_atoms[TARGETS]))
+      selection_data->type == GDK_SELECTION_TYPE_ATOM)
     {
       if (targets)
 	*targets = g_memdup (selection_data->data, selection_data->length);
