@@ -22,50 +22,54 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_STATUSBAR_H__
 #define __GTK_STATUSBAR_H__
 
 #include <gtk/gtkhbox.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_STATUSBAR            (gtk_statusbar_get_type ())
-#define GTK_STATUSBAR(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_STATUSBAR, GtkStatusbar))
-#define GTK_STATUSBAR_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_STATUSBAR, GtkStatusbarClass))
-#define GTK_IS_STATUSBAR(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_STATUSBAR))
-#define GTK_IS_STATUSBAR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_STATUSBAR))
-#define GTK_STATUSBAR_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_STATUSBAR, GtkStatusbarClass))
+#define GTK_STATUSBAR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_STATUSBAR, GtkStatusbar))
+#define GTK_STATUSBAR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_STATUSBAR, GtkStatusbarClass))
+#define GTK_IS_STATUSBAR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_STATUSBAR))
+#define GTK_IS_STATUSBAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_STATUSBAR))
+#define GTK_STATUSBAR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_STATUSBAR, GtkStatusbarClass))
 
 
 typedef struct _GtkStatusbar      GtkStatusbar;
 typedef struct _GtkStatusbarClass GtkStatusbarClass;
-typedef struct _GtkStatusbarMsg GtkStatusbarMsg;
 
 struct _GtkStatusbar
 {
   GtkHBox parent_widget;
 
-  GtkWidget *frame;
-  GtkWidget *label;
+  GtkWidget *GSEAL (frame);
+  GtkWidget *GSEAL (label);
 
-  GSList *messages;
-  GSList *keys;
+  GSList *GSEAL (messages);
+  GSList *GSEAL (keys);
 
-  guint seq_context_id;
-  guint seq_message_id;
+  guint GSEAL (seq_context_id);
+  guint GSEAL (seq_message_id);
+
+  GdkWindow *GSEAL (grip_window);
+
+  guint GSEAL (has_resize_grip) : 1;
 };
 
 struct _GtkStatusbarClass
 {
   GtkHBoxClass parent_class;
 
-  GMemChunk *messages_mem_chunk;
+  gpointer reserved;
 
   void	(*text_pushed)	(GtkStatusbar	*statusbar,
 			 guint		 context_id,
@@ -73,16 +77,16 @@ struct _GtkStatusbarClass
   void	(*text_popped)	(GtkStatusbar	*statusbar,
 			 guint		 context_id,
 			 const gchar	*text);
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
-struct _GtkStatusbarMsg
-{
-  gchar *text;
-  guint context_id;
-  guint message_id;
-};
 
-GtkType    gtk_statusbar_get_type     	(void) G_GNUC_CONST;
+GType      gtk_statusbar_get_type     	(void) G_GNUC_CONST;
 GtkWidget* gtk_statusbar_new          	(void);
 /* If you don't want to use contexts, 0 is a predefined global
  * context_id you can pass to push/pop/remove
@@ -99,9 +103,12 @@ void       gtk_statusbar_remove        	(GtkStatusbar *statusbar,
 					 guint	       context_id,
 					 guint         message_id);
 
+void     gtk_statusbar_set_has_resize_grip (GtkStatusbar *statusbar,
+					    gboolean      setting);
+gboolean gtk_statusbar_get_has_resize_grip (GtkStatusbar *statusbar);
 
+GtkWidget* gtk_statusbar_get_message_area  (GtkStatusbar *statusbar);
+
+G_END_DECLS
 
-#ifdef __cplusplus
-} 
-#endif /* __cplusplus */
 #endif /* __GTK_STATUSBAR_H__ */
