@@ -24,7 +24,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#undef GTK_DISABLE_DEPRECATED
+
+#include "config.h"
 #include <stdio.h>
 #include "gtk/gtk.h"
 
